goto ppp;
@ ------------------------------------------------ Ramsey3d.g --------------------------------------------

   Author: Alfred Maussner
   Date  : 12. December 2003

   Purpose: Solve the stochastic growth model with non-negative investment (Heer/Maussner,
            Chapter 4). You can choose between Chebyshev collocation and the least squared projection method.

   Notes: In order to get proper screen messages, open the DOS compatibility window
          (Tools->DOS Compatibility Window).

          For the program to run you must either include the names of the procedures
          in the file Toolbox.src in your Gauss user library (user.lcg) or in any other
          Gauss library file or use the statement #include Toolbox.src (if this file
          is in your current working directory).

--------------------------------------------------------------------------------------------------------- @

new;
library user, pgraph, dge;

/* Specify the name of the output file */
outfile="d:\\tbk\\tb5\\g\\ch4\\Ramsey3d_25.txt";

/* Parameters of the model */
alpha=0.27;
beta =0.994;
delta=0.011; 
eta  =2.0;   
rho  =0.90;
sigma=0.0072;
sigma=0.05;

/* Parameters of the algorithm */
lsq=0;      @ if 1: least squares, 0 Chebyshev collocation @
znodes=15;  @ number of nodes in Gauss-Chebysehv quadrature with respect to z @
knodes=15;  @ number of nodes in Gauss-Chebysehv quadrature with respect to k @                                  
initial=1;  @ if 1: the matrix gvec0 is loaded, if 0 Search1 is used to find the initial value  @
mca=0;      @ 1: use markov chain approximation, 0: compute integral @
complete=0; @ 1: use complete polynomial, 0: use product base        @
degree1=2;  @ degree of complete polynomial, count from 1 to p       @
degree2=4|4;@ degree of product base polynomial: first row for z, second for k @
nobs=3000;  @ number of valid observations in computation of DM statistic @
ta  =501;   @ number of observations to discard                           @
lags=5;     @ number of lags                                              @
nofs=500;   @ number of simulations                                       @

size=6.0;   @ determine the multiple of the unconditional standard deviation of the process for ln(z) @m=51;      @ number of points of the grid for the productivity shock @
m=101;      @ number of points in zgrid @

/* Boundaries of the state space */
ks=((alpha*beta)/(1-beta*(1-delta)))^(1/(1-alpha)); @ deterministic steady state for the stock of capital     @
cs=ks^alpha - delta*ks;                             @ deterministic steady state for the level of consumption @

kmin=0.2*ks;
kmax=2.0*ks;

zs=0;                                               @ unconditional mean of z @


/* Compute boundaries for productivity shock */
{zgrid,pmat}=MarkovAR(size,m,rho,sigma);            @ grid and transition probability matrix for log of productivity shock @
zmin=zgrid[1];
zmax=zgrid[m];


/* The following variables will be used in the subroutines and must be initialized in
** the main program: zmin, zmax, kmin, kmax, gvec0, zgrid, pmat, FError
*/

clear _gvec;
k0=ks;
z0=zs;
FError=0;

output file=^outfile reset;
Datum;
output off;

/* Initial points */
if initial==1;
 load gvec05_4x4;
 gvec0=gvec05_4x4;
 @gvec0=gvec2[1|2]|0|gvec2[3]|0|0;@

/*  load gvec0072_d2;
**  gvec0=zeros(10,1);  
** gvec0[1:3]=gvec0072_d2[1:3];
**  gvec0[5:6]=gvec0072_d2[4:5];
**  gvec0[8]=gvec0072_d2[6];
*/

else;
   cls;
   npop=50; ngen=50;
   bsec=hsec;
   if complete; 
          npar=(degree1+1)*(degree1+2)/2; 
   else;
          npar=degree2[1]*degree2[2];
   endif;
   if lsq;
      if mca;
          {gvec0,fx}=GSearch1(NPar,Npop,Ngen,&f1);
      else;
          {gvec0,fx}=GSearch1(Npar,Npop,Ngen,&f2);
      endif;
   else;
          {gvec0,fx}=GSearch1(NPar,Npop,Ngen,&f3);      
   endif;
   output on;
   ?"Runtime for GSearch1: " etstr(hsec-bsec);
   output off;  
endif;

/* Find solution */
cls;

bsec=hsec;

if not lsq;
    _MNR_Print=1;
    _MNR_Global=1;
    if mca;
       if complete;
          {gvec2,crit}=FixVMN1(gvec0,&Sys3);     @ complete polynomial and Markov chain approximation @
       else;  
          {gvec2,crit}=FixVMN1(gvec0,&Sys1);     @ product base polynomial and Markov chain approximation    @
       endif;
    else;
       if complete;
          {gvec2,crit}=FixVMN1(gvec0,&Sys4);      @ complete polynomial and GC quadrature of the integral @
       else;
          {gvec2,crit}=FixVMN1(gvec0,&Sys2);      @ product base polynomial and GCquadrature of the integral @
       endif;
    endif;
else;
   _QN_Print=1; 
   _QN_GradTol=1.e-5;  
   if mca;
     {gvec2,crit}=QuasiNewton(gvec0,&f1);
   else;
     {gvec2,crit}=QuasiNewton(gvec0,&f2);
   endif;
endif;

output on;
?"";
if lsq; ?"Runtime for QuasiNewton: "; else; ?"Runtime for Newton-Raphson: "; endif;
?etstr(hsec-bsec); 
?"Termination criterion: ";
crit[1];
if crit[1]==0;
  ?"Found Solution: ";
  if complete; gvec2; else; ?reshape(gvec2,degree2[1],degree2[2]); endif;
  ?"";
  /* Test solution */
  gvecs=gvec2;
  ks1=FixpBS((kmin|kmax),&GetKS);
  ?"Implied stationary stock of capital:      " ks1;
  ?"Deterministic stationary stock of capital:" ks;
  bsec=hsec;output off;
  {dm,freq,bounds,hit,err,sc}=GetDM(gvec2,nobs,nofs,ta,lags);
  output on;
  ?"Run time for simulation: " etstr(hsec-bsec);?"";
  ?"Den Haan/Marcet statistic:";
  if nofs==1; ?dm; endif;
  ?"Percentage of Simulations below lower bound:  " freq[1]/freq[3];
  ?"Percentage of Simulations beyond upper bound: " freq[2]/freq[3];
  ?"Number instances where z or k exceed their boundaries:";
  ?hit;
  ?"average sum of squared errors:             " err;
  ?"average standard deviation of consumption: " sc;
endif;
?"";
?"Parameters:";
?"alpha =   " ftos(alpha,"*.*lf",8,5);
?"beta  =   " ftos(beta ,"*.*lf",8,5);
?"delta =   " ftos(delta,"*.*lf",8,5);
?"eta   =   " ftos(eta,  "*.*lf",8,5);
?"sigma =   " ftos(sigma,  "*.*lf",8,5);
?"rho   =   " ftos(rho,  "*.*lf",8,5);
?"method=   " ftos(lsq,"*.*lf",1,0);
?"Markov=   " ftos(mca,"*.*lf",1,0);
?"complete= " ftos(complete,"*.*lf",1,0);
if complete;
?"degree  = " ftos(degree1,"*.*lf",3,0);
else;
?"degree Z= " ftos(degree2[1],"*.*lf",3,0);
?"degree K= " ftos(degree2[2],"*.*lf",3,0);
endif;
?"m     =   " ftos(m     ,"*.*lf",4,0);
?"size  =   " ftos(size  ,"*.*lf",3,1);
?"nofs  =   " ftos(nofs  ,"*.*lf",4,0);
output off;

/* Compute and plot policy function */
if crit[1]/=0; goto ende; endif;
kvec=seqa(kmin,(kmax-kmin)/(m-1),m);
cmat=zeros(m,m);
i=1;
do until i>m;
   j=1;
   do until j>m;
      cmat[i,j]=GetC(gvec2,zgrid[i],kvec[j]);
              j=j+1;
   endo;
   i=i+1;
endo;
ppp:
Graphsettings;
_plwidth=7;
_pzclr=0;
zlabel("Consumption");
xlabel("Capital");
ylabel("Log of Productivity Shock");
surface(kvec',zgrid,cmat);
ende:
end;


/* ---------------------------------------------- Subroutines --------------------------------------- */

/* GetPsi: Returns the conditional expectation as a function of k=ln(k) and z using a
**        p[1],p[2]-th degree product base Chebyshev polynomial or a p degree complete
**        Chebyshev polynomial. In the first case, the degree must be specified in the variable degree2
**        in the main program. In the second case the degree is specified in the variable degree1.
**        In addition, for the program to know what kind of polynomial to use, the
**        variable complete (defined in the main program) must be stet to 2 or 1.
**
**         Furthermore, the lower and upper bounds for z and k must be
**         given in zmin,zmax, kmin, and kmax in the main program.
*/

proc(1)=GetPsi(gvec,z,k);

    local x, pvec, bounds;

    bounds=(zmin~zmax)|(ln(kmin)~ln(kmax));

    if complete;
       x=Chebeval2(gvec,(z|ln(k)),degree1,bounds);
    else;
       pvec=reshape(gvec,degree2[1],degree2[2]);
          x=Chebeval3(pvec,(z|ln(k)),degree2,bounds);
    endif;
  
    if (x<-1.e16) or (x>400); FError=1; locate 15,1; ?FError; retp(miss(1,1)); endif;
    retp(beta*exp(x));

endp;

/* R1(z,k) returns the residual; for this to work, the vector of parameters gvec
**           must be given in a global variable gvec0. 
**           RSQ1 uses the Markov chain representation of the productivity shock to compute the expectational term  */

proc(1)=R1(z,k);

    local psi,  phi;
   
    z0=z;
    k0=k;
    psi=GetPsi(_gvec,z,k);   if scalmiss(psi); retp(miss(1,1)); endif;
    phi=GetPhi1(z,k);        if scalmiss(phi); retp(miss(1,1)); endif;

  retp(psi-phi);

endp;

/* R2(z,k) returns the  residual; for this to work, the vector of parameters gvec
**          must be given in a global variable gvec0. Also, the boundaries of the z, zmin and
**          zmax must be given. RSQ2 computes the integral to get the expectational term  */

proc(1)=R2(z,k);

    local psi,  phi;
   
    z0=z;
    k0=k;
    psi=GetPsi(_gvec,z,k);            if scalmiss(psi); retp(miss(1,1)); endif;
    phi=GetPhi2(z,k);                     if scalmiss(phi); retp(miss(1,1)); endif;

  retp(psi-phi);

endp;


/* RSQ1(z,k) returns the squared residual as given by R1 */

proc(1)=RSQ1(z,k);

  local r;

  r=R1(z,k);

  retp(r^2);

endp;

/* RSQ2(z,k) returns the squared residual as given by R2 */

proc(1)=RSQ2(z,k);

  local r;
  r=R2(z,k);
 retp(r^2);

endp; 

/* GetPhi1: returns the conditional expectations for given z and k. This is computed
**          from the Markov chain approximation of the stochastic AR(1) process
**          of the technology shock. 
**
*/


proc(1)=GetPhi1(z,k);

 local i,j, x, y, p;

 z0=z;
 k0=k;

 p=armc1(z0,rho,sigma,m,size);

 x=0;
 i=1;
 do until i>m;
    y=PhiZ(zgrid[i])*p[i];
    if scalmiss(y); retp(miss(1,1)); endif;
    x=x+y;
    i=i+1;
 endo;

retp(beta*x);

endp;

/* GetPhi2: Returns the conditional expectation for given z and k.
**          In this procedure, this expression is computed from
**          the integral of p(eps)*(u'(C1)*(1-delta+alhpa*k1^(alpha-1))-(1-delta)*mu1)),
**          where p(eps) is given from the  probability density function
**          of the respective normal distribution 
*/

proc(1)=GetPhi2(z,k);

   local phi;

    z0=z;
    k0=k;
    phi=GC_Int1(&PhiEps,(zmin|zmax),100);  if scalmiss(phi); retp(miss(1,1)); endif;
   retp(beta*phi);

endp;

/* PhiZ: returns the u'(c')(1-delta+f'(k')-(1-delta)*mu', which is involved in the
**       computation of phi(z,k). The program assumes gvec0, k0 and z0 given from the
**       main program.
*/

proc(1)=PhiZ(z);

      local c0, c1, k1,k2, m1, x, psi0, psi1;

      psi0=GetPsi(_gvec,z0,k0);

      if scalmiss(psi0); retp(miss(1,1)); endif;

      /* First Step: Compute k1 */
      c0=psi0^(-1/eta);
      if c0<0; FError=2; locate 15,1; ?FError; retp(miss(1,1)); endif;
      k1=exp(z0)*(k0^alpha)+(1-delta)*k0-c0;
      if (k1<(1-delta)*k0); k1=(1-delta)*k0; endif;
      if ((k1<kmin) or (k1>kmax)); FError=1; retp(miss(1,1)); endif;

      /* Second Step: Compute c1 */
      psi1=GetPsi(_gvec,z,k1);
      if scalmiss(psi1); retp(miss(1,1)); endif;
      
      c1=psi1^(-1/eta);
      if c1<0; FError=3; locate 15,1; ?FError; retp(miss(1,1)); endif;
      k2=exp(z)*(k1^alpha)+(1-delta)*k1-c1;
      
      if (k2<(1-delta)*k1); 
         c1=exp(z)*(k1^alpha);
         m1=c1^(-eta)-psi1;
         if m1<0; FError=4; locate 15,1; ?FError; retp(miss(1,1)); endif;
      else;
         m1=0;
      endif;
      x=(c1^(-eta))*(1-delta + alpha*exp(z)*(k1^(alpha-1)))-(1-delta)*m1;      
    
  retp(x);

endp;


/* PhiEps: returns the integrand in the definition of the conditional expectation on the
**         rhs of the model's Euler equation as a function of epsilion in the transition
**         equation of ln Z:= z, z'=rho*z + epsilon. This is used to compute the
**         conditional expectations via Gauss-Chebyshev quadrature.
**
**         The program assumes the vector of coefficients of the expectations function psi, gvec,
**         as given in the main program, as well as the current values of ln z and the capital stock
**         K. We store those two in z0 and k0.
**         Also, alpha, beta, delta, eta, rho, and sigma are taken from the main program.
*/


proc(1)=PhiEps(Eps);

      local c0, c1, k1, k2, m1, x, p, psi0, psi1;

      psi0=GetPsi(_gvec,z0,k0);
      if scalmiss(psi0); retp(miss(1,1)); endif;

      /* First Step: Compute k1 */
      c0=psi0^(-1/eta);
      if c0<0; FError=2; retp(miss(1,1)); endif;
      
      k1=exp(z0)*(k0^alpha)+(1-delta)*k0-c0;
      
      if (k1<(1-delta)*k0); k1=(1-delta)*k0;  endif;
      if ((k1<kmin) or (k1>kmax)); FError=1; retp(miss(1,1)); endif;

      /* Second Step: Compute c1 */
      psi1=GetPsi(_gvec,rho*z0+eps,k1);
      if scalmiss(psi1); retp(miss(1,1)); endif;
      c1=psi1^(-1/eta);
      k2=exp(rho*z0+eps)*(k1^alpha)+(1-delta)*k1-c1;
      if (k2<(1-delta)*k1); 
         c1=exp(rho*z0+eps)*(k1^alpha);
         m1=c1^(-eta)-psi1;
      else;
         m1=0;
      endif;
      x=(c1^(-eta))*(1-delta + alpha*exp(rho*z0+eps)*(k1^(alpha-1)))-(1-delta)*m1;
      p=((2*pi*(sigma^2))^(-0.5))*(exp((-0.5/(sigma^2))*(eps^2)));

  retp(p*x);

endp;

/* Sys1(gvec): defines the rhs of the system of non-linear equations whose zero is the
**             collocation solution of the model. The degree of the polynomial
**             must be specified in the variable degree in the main program. Uses R1.
*/

proc(1)=Sys1(gvec);

  local zbar,kbar,kt,zt,fx,p,i,j;
  
  fx=zeros(degree2[1],degree2[2]);
  p=degree2[1];  
  zbar=cos(((2*seqa(1,1,p)-1)/(2*p))*pi);
  p=degree2[2];
  kbar=cos(((2*seqa(1,1,p)-1)/(2*p))*pi);
  kt=0.5*(kbar+1)*(kmax-kmin) + kmin;      @ transformed to k in [kmin,kmax] @
  zt=0.5*(zbar+1)*(zmax-zmin) + zmin;
  _gvec=gvec;  
  i=1;
  do until i>degree2[1];
     j=1;
     do until j>degree2[2];
        fx[i,j]=R1(zt[i],kt[j]);
        j=j+1;        
     endo;
     i=i+1;
  endo;

retp(vec(fx));

endp;

/* Sys2(gvec): defines the rhs of the system of non-linear equations whose zero is the
**             collocation solution of the model. The degree of the polynomial
**             must be specified in the variable degree in the main program. Uses R2.
*/

proc(1)=Sys2(gvec);

  local zbar,kbar,kt,zt,fx,p,i,j;
  
  fx=zeros(degree2[1],degree2[2]);
  p=degree2[1];     
  zbar=cos(((2*seqa(1,1,p)-1)/(2*p))*pi);
  p=degree2[2]; 
  kbar=cos(((2*seqa(1,1,p)-1)/(2*p))*pi);
  kt=0.5*(kbar+1)*(kmax-kmin) + kmin;      @ transformed to k in [kmin,kmax] @
  zt=0.5*(zbar+1)*(zmax-zmin) + zmin;

  _gvec=gvec;
  
  i=1;
  do until i>degree2[1];
     j=1;
     do until j>degree2[2];
        fx[i,j]=R2(zt[i],kt[j]);
        j=j+1;        
     endo;
     i=i+1;
  endo;

retp(vec(fx));

endp;

/* Sys3(gvec): the same as Sys1, but for the complete polynomial case. */

proc(1)=Sys3(gvec);

  local zbar,kbar,kt,zt,fx,p,i,k,j;
  p=(degree1+1)*(degree1+2)/2;   
  fx=zeros(p,1);
  p=degree1+1;
  zbar=cos(((2*seqa(1,1,p)-1)/(2*p))*pi);
  kbar=cos(((2*seqa(1,1,p)-1)/(2*p))*pi);
  kt=0.5*(kbar+1)*(kmax-kmin) + kmin;      @ transformed to k in [kmin,kmax] @
  zt=0.5*(zbar+1)*(zmax-zmin) + zmin;
  _gvec=gvec;
  p=degree1+1;
  i=1;k=1;
  do until i>p;
     j=1;
     do until j>(p+1-i);
        fx[k]=R1(zt[j],kt[i]);
        j=j+1;
        k=k+1;
     endo;
     i=i+1;
  endo;

retp(fx);

endp;


/* Sys4(gvec): the same as Sys2, but for the complete polynomial case. */

proc(1)=Sys4(gvec);

  local zbar,kbar,kt,zt,fx,p,i,k,j;
  p=(degree1+1)*(degree1+2)/2;   
  fx=zeros(p,1);
  p=degree1+1;
  zbar=cos(((2*seqa(1,1,p)-1)/(2*p))*pi);
  kbar=cos(((2*seqa(1,1,p)-1)/(2*p))*pi);
  kt=0.5*(kbar+1)*(kmax-kmin) + kmin;      @ transformed to k in [kmin,kmax] @
  zt=0.5*(zbar+1)*(zmax-zmin) + zmin;
  _gvec=gvec;
  p=degree1+1;
  i=1;k=1;
  do until i>p;
     j=1;
     do until j>(p+1-i);
        fx[k]=R2(zt[j],kt[i]);
        j=j+1;
        k=k+1;
     endo;
     i=i+1;
  endo;

retp(fx);

endp;



/* f1(gvec): the minimizer of this function is the least squares approximation of the conditional expectations function
**
**           From the main program: zmin,zmax, kmin, and kmax must be known */

proc(1)=f1(gvec);

   local fx, d, n;

   _gvec=gvec;
   d=(zmin~zmax)|(kmin~kmax);
   n=znodes|knodes;

   FError=0; @ clear Error flag @

   fx=GC_Int2(&RSQ1,d,n);

retp(fx);

endp;

/* f2(gvec): the minimizer of this function is the least squares approximation of the conditional expectations function
**           in the case of method=2
**
**           From the main program: zmin,zmax, kmin, and kmax must be known */

proc(1)=f2(gvec);

   local fx, d, n;

   _gvec=gvec;
   d=(zmin~zmax)|(kmin~kmax);
   n=znodes|knodes;

   FError=0; @ clear Error flag @

   fx=GC_Int2(&RSQ2,d,n);

retp(fx);

endp;

/* f3: used for GSearch1 to find a good starting value for Chebyshev collocation,
**     it just returns the Euklidian norm of Sys1
*/

proc(1)=f3(gvec);

   local fx;
   if complete;
      if mca;
        fx=Sys3(gvec);
      else;
        fx=Sys4(gvec);
      endif;
   else;
      if mca;
        fx=Sys1(gvec);
      else;
        fx=Sys2(gvec);
      endif;
   endif;

retp(sqrt(fx'fx));

endp;


/* The zero of the rhs returned by this procedure gives the stationary
** stock of capital.
*/

proc(1)=GetKS(k);

  local psi, c1, c2;
  psi=GetPsi(gvecs,zs,k);
  c1=psi^(-1/eta);
  c2=exp(zs)*(k^alpha)-delta*k;

retp(c1-c2);

endp;

/* ARMC1: Computes the transition probability from
**        state x0 to state x[i].
**
*/

proc armc1(x0,a,s2,NN,lambda);
    local f1,f2,p,sy,w,y;

    sy=s2/sqrt(1-a^2);
    y=seqa(-lambda*sy,(2*lambda*sy)/(NN-1),NN);    
    w=y[2]-y[1];
    f1=0~cdfn((y[2:NN]'-a*x0-w/2)/s2);
    f2=cdfn((y[1:NN-1]'-a*x0+w/2)/s2)~1;
    p=f2-f1;
    retp(p);

endp;


/* GetDM: Computes the DM statistic */

proc(6)=GetDM(gvec,nobs,nofs,ta,lags);

  local zt, kt, ct, mt, tmax, i, j, t, freq, psi,
        et,zmat,avec,amat,dm,bounds,epst,hit, errsum, sc;

  /* Compute a time series of length nobs */
  errsum=zeros(nofs,1);
  sc=zeros(nofs,1);
  hit=zeros(2,2);
  tmax=nobs+ta+lags+1;
  freq=zeros(3,1);
  dm=zeros(nofs,1);
  bounds=zeros(1,2);
  bounds[1]=cdfchii(0.025,1+2*lags);
  bounds[2]=cdfchii(0.975,1+2*lags);
  _gvec=gvec;
  i=1;
  do until i>nofs;  @ loop over number of simulations @
    locate 23,2; ?"Simulation No. " ftos(i,"*.*lf",4,0);
    if mca;
      zt=zgrid[MarkovTP(tmax,pmat,floor(m/2))];  @ compute vector of shocks @
    else;
      zt=zeros(tmax+1,1);
    epst=sigma*rndn(tmax,1);
      t=1;
      do until t>tmax;
         zt[t+1]=rho*zt[t]+epst[t];
               t=t+1;
      endo;
      if maxc(zt)>zmax; hit[1,2]=hit[1,2]+1; endif;
      if minc(zt)<zmin; hit[1,1]=hit[1,1]+1; endif;
    endif;
      kt=zeros(tmax+1,1); kt[1]=ks;
      ct=zeros(tmax,1);
      mt=zeros(tmax,1);
      t=1;
     do until t>tmax;
        psi=GetPsi(gvec,zt[t],kt[t]);
        ct[t]=psi^(-1/eta);
      kt[t+1]=exp(zt[t])*(kt[t]^alpha)+(1-delta)*kt[t]-ct[t];

      /* Kuhn-Tucker conditions */
      if (kt[t+1]<(1-delta)*kt[t]);
         kt[t+1]=(1-delta)*kt[t];
         ct[t]  = exp(zt[t])*(kt[t]^alpha);
         mt[t]  = ct[t]^(-eta) - psi;
      else;
         mt[t]  = 0;
      endif;
       
      t=t+1;
     endo;
     freq[3]=freq[3]+1;
     if maxc(kt)>kmax; hit[2,2]=hit[2,2]+1; endif;
     if minc(kt)<kmin; hit[2,1]=hit[2,1]+1; endif;
     
/* Compute DM statistic */
     t=ta+lags;
     et=zeros(nobs,1);
   tmax=tmax-2;
   zmat=ones(nobs,1+2*lags);
     do until t>tmax;
       /* if mca; 
           errsum[i]=errsum[i]+RSQ1(zt[t],kt[t]);
        else;
           errsum[i]=errsum[i]+RSQ2(zt[t],kt[t]);
        endif;
      */
      
        et[t-ta-lags+1]=beta*(ct[t+1]^(-eta))*(1-delta+alpha*exp(zt[t+1])*(kt[t+1]^(alpha-1))) - beta*(1-delta)*mt[t+1]-
                        (ct[t]^(-eta))+mt[t];        
        j=1;
        do until j>lags;
                zmat[t-ta-lags+1,1+j]=ct[t-j];
           zmat[t-ta-lags+1,1+lags+j]=zt[t-j];
                                    j=j+1;
        endo;
        t=t+1;
     endo;
     errsum[i]=errsum[i]/nobs;
     sc[i]=stdc(ct[ta+lags:tmax-2]);
     avec=zmat'*et;
     amat=zeros(2*lags+1,2*lags+1);
        t=1;
     do until t>nobs;
        amat=amat+(zmat[t,.]'zmat[t,.])*et[t]^2;
           t=t+1;
     endo;
     dm[i]=avec'*invpd(amat)*avec;
     
     if (dm[i] < bounds[1]); freq[1]=freq[1]+1; endif;
     if (dm[i] > bounds[2]); freq[2]=freq[2]+1; endif;

    i=i+1;

  endo;

retp(dm,freq,bounds,hit,meanc(errsum),meanc(sc));

endp;

/* BoundsTest: Computes the integral over the pdf of the normal
**             distribution with mean 0 and var sigma^2 between
**             bounds[1] and bounds[2] using Gauss-Chebyshev
**             quadrature.
*/

proc(1)=BoundsTest(bounds);
  
local phi;
  
  phi=GC_Int1(&Normal,(bounds[1]|bounds[2]),100);

retp(phi);

endp;

/* Normal: returns the pdf of the normal with mean 0 and variance
**         sigma^2
*/

proc(1)=Normal(eps);

  retp(((2*pi*(sigma^2))^(-0.5))*(exp((-0.5/(sigma^2))*(eps^2))));  

endp;

/* GetC: Computes the policy function C(gvec,k[i],z[j]); */

proc(1)=GetC(gvec,z,k);

   local psi, k1, c;

   psi=GetPsi(gvec,z,k);

    c=psi^(-1/eta);
   k1=exp(z)*(k^alpha)+(1-delta)*k-c;
   if (k1<(1-delta)*k); c=exp(z)*(k^alpha); endif;

 retp(c);

endp;


proc(0)=Datum;

  local t,d,str1,str2;
  d=date;
  t=time;
  str1=ftos(d[3],"%0*.*lf",2,0) $+ "." $+ ftos(d[2],"%0*.*lf",2,0) $+"." $+ ftos(d[1],"%0*.*lf",2,0);
  str2=ftos(t[1],"%*.*lf",2,0)  $+ ":" $+ ftos(t[2],"%0*.*lf",2,0) $+":" $+ ftos(t[3],"%0*.*lf",2,0);
  ?str1;
  ?str2;
retp;

endp;
